import kCollection from '../../src/kCollection'

describe('kCollection/replaceWith.js', () => {
  describe('replaceWith', () => {
    beforeEach(() => {
      document.body.innerHTML =`
        <div class="item">Some</div>
        <div class="item">Item</div>
        <span class="itemReplacement">Hello</span>
        <span class="itemReplacement">World!</span>`
    })
    it('when calling replaceWith() on an empty collection, should return an empty collection', () => {
      const collection = kCollection().replaceWith(kCollection())
      expect(collection.get()).toHaveLength(0)
    })
    it('when calling replaceWith() width an empty collection, should return an empty collection', () => {
      const toReplace = document.querySelectorAll('.item')
      const collection = kCollection(toReplace).replaceWith(kCollection())
      expect(collection.get()).toHaveLength(0)
    })
    it('when calling replaceWith() with a collection, should return given collection', () => {
      const toReplace = document.querySelectorAll('.item')
      const replaceWith = document.querySelectorAll('.itemReplacement')
      const replacementCollection = kCollection(replaceWith)
      const resultCollection =  kCollection(toReplace).replaceWith(replacementCollection)
      expect(resultCollection).toBe(replacementCollection)
    })
    it('when calling replaceWith() on a collection, should replace (move) nodes in DOM', () => {
      const toReplace = document.querySelectorAll('.item')
      const replaceWith = document.querySelectorAll('.itemReplacement')
      kCollection(toReplace).replaceWith(kCollection(replaceWith))
      expect(document.body.innerHTML).toBe(`
        
        <span class="itemReplacement">Hello</span><span class="itemReplacement">World!</span>
        
        `
      )
    })
    it('when calling replaceWith() width an empty collection, should remove nodes in DOM', () => {
      const toReplace = document.querySelectorAll('.item')
      kCollection(toReplace).replaceWith(kCollection())
      expect(document.body.innerHTML).toBe(`
        
        
        <span class="itemReplacement">Hello</span>
        <span class="itemReplacement">World!</span>`
      )
    })
    it('when calling replaceWith() without a collection argument, should throw an error', () => {
      const toReplace = document.querySelectorAll('.item')
      expect(() =>  kCollection(toReplace).replaceWith('notacollection')).toThrow()
    })
  })
})
